// if you want to skip the messages on the serial monitor comment the macro below
#define  _DEBUG_MESSAGE

#define  BUTTON_PRESSED  (!(PINE & 0x04))
#define  LED_OFF     0
#define  LED_ON      1
#define  LED_TOGGLE  2

void Connect_to_WiFi_Network (char Net[], char Pass[])
{
  String Command;
  Command = "AT+CWJAP=\"";
  Command = Command + Net;
  Command = Command + "\",\"";
  Command = Command + Pass;
  Command = Command + "\"\r";
  Serial1.print (Command);
}

void Disconnect_from_WiFi_Network (void)
{
  Serial1.print ("AT+CWQAP\r");
}

// using PIC-WEB as a server
// the default server IP address is 192.168.0.130
// if you want to change it edit the macro below
#define  IP_Server  "192.168.0.130"
void Get_Server_Page ()
{
  String Cmd, IP;
  long int Timeout = 10000000;
  IP = "AT+CIPSTART=\"TCP\",\"";
  IP += IP_Server;
  IP += "\",80";
  Serial1.println (IP);
  Serial.println (IP);  // debug purpose
  if (Serial1.find ("Error"))
    return;

  // by default index.htm (/<empty>) otherwise should be the subpage name
  // for example ("GET /forms.htm HTTP/1.0\r\n\r\n"
  Cmd = "GET / HTTP/1.0\r\n\r\n";

  Serial1.print("AT+CIPSEND=");
  Serial1.println (Cmd.length());
  if (Serial1.find (">"))
  {
    Serial1.print(Cmd);
    Serial.print ('>');
  }
  else
  {
    Serial1.println ("AT+CIPCLOSE");
    Serial.println ("Connection Timeout!");
    delay (1000);
    return;
  }
  
  
  // original but delay is too long and during that time the data
  // sent by the server to the ESP module are lost;
  // replaced by timeout instead with value 100K
  // updated after each character
  //delay (2000);

  while (Timeout--)
  {
    static bool FirstTime=true;
    int Number=0, i=0;

    // filtering metadata so only page data will be printed
    if (FirstTime)
      FirstTime=false;
    else
      Serial1.find ("OK");

    Serial1.find ("+IPD,");
    delay (1);
    while (1)
    {
      char c=0;
      if (Serial1.available())
      {
        c = Serial1.read ();
        if (c == ':')
          break;
        else
          Number = Number * 10 + (c & 0x0F);
      }
      else
      {
        Timeout--;
        if (!Timeout)
          break;
      }
    }
    for (i=0; i<Number;)
      if (Serial1.available ())
      {
        Serial.print ((char)Serial1.read ());
        Timeout = 100000;
        i++;
      }
  }
  Serial.println ("Transfer complete!");
}

void setup()
{
  // LEDs on Olimexino set as outputs
  pinMode (7, OUTPUT);
  digitalWrite (7, LOW);
  pinMode (9, OUTPUT);
  digitalWrite (9, LOW);

  DDRE = DDRE & (~(1<<2));  // Button - input 

  pinMode (8, OUTPUT);    // UEXT Power
  digitalWrite (8, LOW);  // UEXT Power turned on
  
  #ifdef  _DEBUG_MESSAGE
  Serial.begin (115200);
  #endif
  Serial1.begin (115200);

  delay (1000);
  Serial1.print ("AT+CWMODE=3\r");
  delay (1000);
  Serial1.print ("AT+RST\r");

  Disconnect_from_WiFi_Network ();
  Connect_to_WiFi_Network ("OlimexTenda", "DarmadaniaSarsapat");
}

bool Button = true;  // this variable is used to avoid command repetition due to holding the button
int Val=0;

void loop ()
{
  if (BUTTON_PRESSED && Button)
  {
    // toggling LEDs on the Olimexino board indicates that button is pressed
    digitalWrite (7, Val);
    digitalWrite (9, !Val);
    Val = !Val;

    // sending command to the server
    Get_Server_Page ();

    Button = false;  // mark button as pressed
  }

  if (!BUTTON_PRESSED)
    Button = true;  // mark button as released
}

